/*
 * Copyright(C) OASIS(R) 2009, 2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 */
package client;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.Properties;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;

import org.apache.tuscany.sca.node.Node;
import org.apache.tuscany.sca.node.equinox.launcher.Contribution;
import org.apache.tuscany.sca.node.equinox.launcher.NodeLauncher;

/**
 * An implementation of the Runtime Bridge for the Apache Tuscany SCA runtime (version 2.x)
 * @author Kelvin Goodson
 *
 */
public class TuscanyRuntimeBridge implements RuntimeBridge {
	
	static final String CONTRIBUTION_LOCATION_PROPKEY = "OASIS_TESTENV_CONTRIBUTION_LOCATION";
	
    protected NodeLauncher launcher;
    protected Node node;
    protected Properties expectedErrorMessages;
    TestConfiguration testConfiguration = null;
    
    public TuscanyRuntimeBridge() {
        // read test error mapping
        expectedErrorMessages = new Properties();
        try {
            InputStream propertiesStream = this.getClass().getResourceAsStream("/tuscany-oasis-sca-tests-errors.properties");
            expectedErrorMessages.load(propertiesStream);
        } catch (IOException e) {   
            System.out.println("Unable to read oasis-sca-tests-errors.properties file");
        } 
    }
        
//	public TestConfiguration getTestConfiguration() {
//		return testConfiguration;
//	}

//	public void setTestConfiguration(TestConfiguration testConfiguration) {
//		this.testConfiguration = testConfiguration;
//	}

	public boolean startContribution(String compositeName,
			                         String contributionLocation,
			                         String[] contributionNames) throws Exception {
		try {
		// Tuscany specific code which starts the contribution(s) holding the test
        launcher = NodeLauncher.newInstance();

        Contribution[] contributions = new Contribution[contributionNames.length];
        String[] contributionURIs = getContributionURIs( contributionLocation, contributionNames );
        for( int i = 0; i < contributions.length ; i++ ) { 
        	contributions[i] = new Contribution(contributionNames[i], contributionURIs[i]);
        } // end for
        
		node = launcher.createNode(compositeName, contributions );
        // Start the node
        node.start();
		} catch (Exception e) {
			System.out.println( "Exception during startup: " + e.getMessage() );
			//e.printStackTrace();
			throw e;
		} // end try

        return true;
	} // end method startContribution

	/**
     * Gets the location of the Contributions as URIs
     * @param contributionLocation - a location pattern URI, which contains one or more "%1"
     * substrings, each of which are substituted with the name of the contribution to get the URI of
     * the contribution
     * @param contributionNames - an array of contribution names
     * @return the contribution location URIs as an array of Strings
     */
    protected String[] getContributionURIs(String contributionLocation, String[] contributionNames) throws Exception {
    	String[] locations = new String[contributionNames.length];
    	
    	if( locations != null && contributionLocation != null ) {
	    	  	
    		for( int i=0; i < locations.length; i++ ) {
    			String aLocation = contributionLocation.replaceAll("%1", contributionNames[i]);
		    			    
		    	locations[i] = aLocation;
    		} // end for    	  	
    	} else {
    		if( locations == null ) {
        		// No contribution specified - throw an Exception
    			throw new Exception("Unable to start SCA runtime - no contribution supplied - error");
    		} else {
    			// No contribution location supplied - throw an Exception
    			throw new Exception("Unable to start SCA runtime - no contribution location supplied - error");
    		} // end if 
    	} // end if
	    	
    	return locations;
    } // end getContributionURI
    
	public void stopContribution() {
        if (node != null) {
            node.stop();
        } // end if
        if (launcher != null) {
            launcher.destroy();
        } // end if
	} // end method stopContribution
	
//	public String getContributionLocation(Class<?> testClass) {
//    	return ContributionLocationHelper.getContributionLocation(testConfiguration.getTestClass());
//	} // end method getContributionLocation

	public void checkError(String testName, Throwable ex) throws Throwable {       
        String expectedMessage = expectedErrorMessages.getProperty(testName);
        String receivedMessage = ex.getMessage();
        
        if (expectedMessage == null){
            writeMissingMessage(testName, ex);
            fail("Null expected error message for test " + testName + 
            	 "Please add message to oasis-sca-tests-errors.properties");
        } // end if
        
        if (receivedMessage == null){
        	ex.printStackTrace();
            fail("Null received error message for test " + testName);
        } // end if

        if (expectedMessage.startsWith("*")) {
            // allow using * to ignore a message comparison
            return;
        }
        
        // Deal with the case where the message has variable parts within it
        // marked with the characters ***. Here we tokenize the epected string 
        // and make sure all the individual parts are present in the results string
        String expectedMessageParts[] = expectedMessage.split("\\*\\*\\*");
        
        if (expectedMessageParts.length > 1){
            int foundParts = 0;
            for(int i = 0; i < expectedMessageParts.length; i++){
                if (receivedMessage.indexOf(expectedMessageParts[i]) > -1 ){
                    foundParts++;
                }
            }
            
            if (foundParts == expectedMessageParts.length){
                return;
            }
        }

        
        // Deal with the case where the end of the message is variable (eg contains absolute filenames) 
        // and where the only relevant part is the start of the message - in this case the expected
        // message only contains the stem section which is unchanging...
        if( receivedMessage.length() > expectedMessage.length() ) {
        	// Truncate the received message to the length of the expected message
        	receivedMessage = receivedMessage.substring(0, expectedMessage.length() );
        } // end if
        if (!expectedMessage.equals(receivedMessage)) {
            writeIncorrectMessage(testName, expectedMessage, receivedMessage);
        }
        
        assertEquals( expectedMessage, receivedMessage );
        
        return;
    } // end method checkError
	
    protected void writeMissingMessage(String testName, Throwable ex) {
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter("target/OTestMissingMsgs.txt", true));
            out.write(testName + "=*");
            out.newLine();
            out.close();
        } catch (IOException e) {
        } 
    }

    protected void writeIncorrectMessage(String testName, String expected, String received) {
        try {
            BufferedWriter out = new BufferedWriter(new FileWriter("target/OTestIncorrectMsgs.txt", true));
            out.write(testName); out.newLine();
            out.write("    " + expected); out.newLine();
            out.write("    " + received); out.newLine();
            out.close();
        } catch (IOException e) {
        } 
    }
} // end class TuscanyRuntimeBridge
